unit MDIChildFormControl;
{.$define debug}


interface
uses Forms, Dialogs, Controls, Classes, sysUtils;

Const
  MyFavoriteComponentTab='GWT';

Type
  // to be placed on the main form only.

  TMDIForm=Class(TComponent)
  private
    fMaxInstances:Integer; //Maximum allowed instances
    fInstances:Integer;    //Current number of instances
    fOnFormCreate:TNotifyEvent;
    fOnFormDestroy:TNotifyEvent;
    Procedure SetMaxInstances(Value:integer);
  Public

    // this method is to be used in place of
    // application.createform(TMyform,Myform);
    // Use as follows
    // Myform:=MDIChildFormControl.Open(TMyForm);
    Function Open(NewForm:TFormClass):TObject;
    constructor Create(AOwner:TComponent); override;
  published
    property MaxInstances:Integer read fMaxInstances write fMaxInstances default 1;
  end;

  // Place on an MDIChildForm.  Wraps the owner form onclose event
  // so that it defaults to action:=cafree;
  // This way it is not necessary to set the onclose event of every
  // MDIchild form when the only thing it will do is set action to caFree;
  // Property IsDialog.  When set to true
  // Causes the form to act as a dialog box.  i.e.
  // it is not resizeable.  Does nothing when set to false
  // Any form properties that were adjusted previously will need to be
  // reset manually at design time.

  TMDIChild=Class(TComponent)
  private
    fIsDialog:Boolean;
    fOnFormCreate:TNotifyEvent; // not used.
    fOnFormclose:TCloseEvent;
    Procedure SetIsDialog(Value:Boolean);
    // wrapper event for form onclose event.  Mainly to set
    // Action to default to caFree.  Action can be overridden in the
    // standard onclose event to anything else you like.
    procedure mdiClose(Sender: TObject; var Action: TCloseAction);
  Protected
    Procedure Loaded; Override;
  Public

    // this method is to be used in place of
    // application.createform(TMyform,Myform);
    // Use as follows
    // Myform:=MDIChildFormControl.Open(TMyForm);
    constructor Create(AOwner:TComponent); override;
    destructor destroy; override;
  published
    //Prevents form from being resized if it is desired to treat it as a
    //dialog while still an mdichild.
    property IsDialog:Boolean read fIsDialog write SetIsDialog;
  end;

  Procedure Register;

implementation
{$ifdef debug}
uses
  commonfunctions;
{$endif}


Procedure Register;
begin
  RegisterComponents(MyFavoriteComponentTab, [TMDIForm,TMDIChild]);
end;

//******************************************************************************
//
//                       TMDIChild Component
//
//******************************************************************************

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIChild.SetIsDialog
//
////////////////////////////////////////////////////////////////////////////////

Procedure TMDIChild.SetIsDialog(Value:Boolean);
begin
  if Value then
  begin
    (Owner as TForm).BorderIcons:=[biSystemMenu];
    (Owner as TForm).BorderStyle:=bsSingle;
  end;
  fIsDialog:=Value;
end;

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIChild.mdiClose
//
////////////////////////////////////////////////////////////////////////////////

procedure TMDIChild.mdiClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
  if assigned(fOnFormClose) then
    fOnFormClose(Sender,action);
end;

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIChild.Loaded
//
////////////////////////////////////////////////////////////////////////////////

Procedure TMDIChild.Loaded;
begin
{$ifdef debug}
  try
{$endif debug}
  inherited;
  With (Owner as TForm) do
  begin
    if (not (csDesigning In ComponentState)) and
       assigned(OnClose) then
      fOnFormClose:=OnClose;
    OnClose:=mdiClose;
  end;
{$ifdef debug}
  except
    trapException('Loaded failed '+className);
  end;
{$endif debug}
end;

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIChild.Create
//
////////////////////////////////////////////////////////////////////////////////

constructor TMDIChild.Create(AOwner:TComponent);
var
  i:integer;
  Exists:boolean;
begin
{$ifdef debug}
  try
{$endif debug}
  Inherited;
  Exists:=false;
  if Owner is TForm then
  with (Owner as TForm) do
  begin
//    WindowState:=wsMinimized;
    if (csDesigning In ComponentState) then
    begin
      for i:=0 to componentcount-1 do
      begin
        if (components[i].classname=self.classname) and
           exists then
          raise Exception.Create('Component of type '+self.Classname+
                                 'already exists on this form');
        if components[i].classname=self.classname then
          exists:=true;
      end;
      if FormStyle<>fsMDIChild then
      begin
        if MessageDlg('FormStyle is not set to fsMDIChild.'+#13+#10+
                      'Change it?', mtWarning, [mbYes, mbNo], 0)=mrYes then
          FormStyle:=fsMDIChild
        else
          raise Exception.Create('Form must be fsMDIChild for this component');
      end;
    end;
    if fIsDialog then
    begin
      BorderIcons:=[biSystemMenu];
      BorderStyle:=bsSingle;
    end;
  end;  //with owner as TForm
{$ifdef debug}
  except
    trapException('Create failed '+className);
  end;
{$endif debug}
end;

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIChild.Destroy
//
////////////////////////////////////////////////////////////////////////////////

Destructor TMDIChild.Destroy;
begin
{$ifdef debug}
  try
{$endif debug}
  if assigned(fOnFormClose) and assigned(owner) then
    with (Owner as TForm) do
      OnClose:=fOnFormClose;
  inherited;
{$ifdef debug}
  except
    trapException('Destroy failed '+className);
  end;
{$endif debug}
end;

//******************************************************************************
//
//                       TMDIForm Component
//
//******************************************************************************

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIForm.Create
//
////////////////////////////////////////////////////////////////////////////////

constructor TMDIForm.Create(AOwner:TComponent);
var
  i:integer;
  Exists:boolean;
begin
{$ifdef debug}
  try
{$endif debug}
  Inherited;
  Exists:=false;
  if fMaxInstances=0 then
    fMaxInstances:=1;
  if Owner is TForm then
  with (Owner as TForm) do
  begin
//    WindowState:=wsMinimized;
    for i:=0 to componentcount-1 do
    begin
      if (components[i].classname=self.classname) and
         exists then
        raise Exception.Create('Component of type '+self.Classname+
                               'already exists on this form');
      if components[i].classname=self.classname then
        exists:=true;
    end;
    if FormStyle<>fsMDIForm then
    begin
      if MessageDlg('FormStyle is not set to fsMDIForm.'+#13+#10+
                    'Change it?', mtWarning, [mbYes, mbNo], 0)=mrYes then
        FormStyle:=fsMDIForm
      else
        raise Exception.Create('Form must be MDIForm for this component');
    end;
  end;
{$ifdef debug}
  except
    trapException('Create failed '+className);
  end;
{$endif debug}
end;

////////////////////////////////////////////////////////////////////////////////
//
//                       TMDIForm.SetMaxInstances
//
////////////////////////////////////////////////////////////////////////////////

procedure TMDIForm.SetMaxInstances(Value:integer);
begin
    fMaxInstances:=Value;
end;

////////////////////////////////////////////////////////////////////////////////

Function TMDIForm.Open(NewForm:TFormClass):TObject;
var
  CurrentInstances,
  i:integer;
  CurrentName:String;
begin
{$ifdef debug}
  try
{$endif debug}
  CurrentName:=NewForm.ClassName;
  Result:=nil;
  CurrentInstances:=0;
  if not (csdesigning in ComponentState) then
  for i:=0 to Screen.FormCount-1 do
  begin
    if Screen.Forms[i].ClassName=CurrentName then
      inc(CurrentInstances);
    if CurrentInstances>=MaxInstances then
    begin
      if Screen.Forms[i].ClassName=CurrentName then
      begin
        Screen.Forms[i].BringToFront;
        Screen.Forms[i].WindowState:=wsnormal;
        Result:=Screen.Forms[i];
      end;
      exit;
    end;
  end;//for
  // if we get here then let's create this form
  Application.CreateForm(NewForm,Result);
{$ifdef debug}
  except
    trapException('Open failed '+className);
  end;
{$endif debug}
end;

end.
